package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.MaintenanceTask;
import org.eco.als.domain.bo.MaintenanceTaskBo;
import org.eco.als.domain.vo.MaintenanceTaskVo;
import org.eco.als.mapper.MaintenanceTaskMapper;
import org.eco.als.service.IMaintenanceTaskService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.MaintenanceTaskTableDef.MAINTENANCE_TASK;

/**
 * 维修任务Service业务层处理
 *
 * @author wgk
 * @date 2024-07-24
 */
@Service
@Slf4j
public class MaintenanceTaskServiceImpl extends BaseServiceImpl<MaintenanceTaskMapper, MaintenanceTask> implements IMaintenanceTaskService {
    @Resource
    private MaintenanceTaskMapper maintenanceTaskMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(MAINTENANCE_TASK);
    }

    private QueryWrapper buildQueryWrapper(MaintenanceTaskBo maintenanceTaskBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(MAINTENANCE_TASK.SORTIE_NO.eq
            (maintenanceTaskBo.getSortieNo()));
        queryWrapper.and(MAINTENANCE_TASK.AIRCRAFT_NO.eq
            (maintenanceTaskBo.getAircraftNo()));
        queryWrapper.and(MAINTENANCE_TASK.SYSTEM_NAME.like
            (maintenanceTaskBo.getSystemName()));
        queryWrapper.and(MAINTENANCE_TASK.UNIT_NAME.like
            (maintenanceTaskBo.getUnitName()));
        queryWrapper.and(MAINTENANCE_TASK.HANDLER_DATE.eq
            (maintenanceTaskBo.getHandlerDate()));
        queryWrapper.and(MAINTENANCE_TASK.STATUS.eq
            (maintenanceTaskBo.getStatus()));

        return queryWrapper;
    }

    /**
     * 查询维修任务
     *
     * @param id 维修任务主键
     * @return 维修任务
     */
    @Override
    public MaintenanceTaskVo selectById(Long id) {
        return this.getOneAs(query().where(MAINTENANCE_TASK.ID.eq(id)), MaintenanceTaskVo.class);

    }


    /**
     * 查询维修任务列表
     *
     * @param maintenanceTaskBo 维修任务Bo
     * @return 维修任务集合
     */
    @Override
    public List<MaintenanceTaskVo> selectList(MaintenanceTaskBo maintenanceTaskBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(maintenanceTaskBo);
        return this.listAs(queryWrapper, MaintenanceTaskVo.class);
    }

    /**
     * 分页查询维修任务列表
     *
     * @param maintenanceTaskBo 维修任务Bo
     * @return 分页维修任务集合
     */
    @Override
    public PageResult<MaintenanceTaskVo> selectPage(MaintenanceTaskBo maintenanceTaskBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(maintenanceTaskBo);
        Page<MaintenanceTaskVo> page = this.pageAs(PageQuery.build(), queryWrapper, MaintenanceTaskVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增维修任务
     *
     * @param maintenanceTaskBo 维修任务Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(MaintenanceTaskBo maintenanceTaskBo) {
        MaintenanceTask maintenanceTask = MapstructUtils.convert(maintenanceTaskBo, MaintenanceTask.class);

        return this.save(maintenanceTask);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增维修任务，前台提供主键值，一般用于导入的场合
     *
     * @param maintenanceTaskBo 维修任务Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(MaintenanceTaskBo maintenanceTaskBo) {
        MaintenanceTask maintenanceTask = MapstructUtils.convert(maintenanceTaskBo, MaintenanceTask.class);


        return maintenanceTaskMapper.insertWithPk(maintenanceTask) > 0;//前台传来主键值
    }

    /**
     * 修改维修任务
     *
     * @param maintenanceTaskBo 维修任务Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(MaintenanceTaskBo maintenanceTaskBo) {
        MaintenanceTask maintenanceTask = MapstructUtils.convert(maintenanceTaskBo, MaintenanceTask.class);
        if (ObjectUtil.isNotNull(maintenanceTask) && ObjectUtil.isNotNull(maintenanceTask.getId())) {
            return this.updateById(maintenanceTask);
        }
        return false;
    }

    @Override
    public void asyncExport(List<MaintenanceTaskVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, MaintenanceTaskVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除维修任务
     *
     * @param ids 需要删除的维修任务主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
